@@ -1,5 +1,6 @@
 """Wrapper around CTGAN model."""
 
+import numpy as np
 from ctgan import CTGANSynthesizer, TVAESynthesizer
 
 from sdv.tabular.base import BaseTabularModel
@@ -18,8 +19,6 @@ class CTGANModel(BaseTabularModel):
         'O': 'label_encoding'
     }
 
-    _cuda = True
-
     def _build_model(self):
         return self._MODEL_CLASS(**self._model_kwargs)
 
@@ -32,39 +31,53 @@ class CTGANModel(BaseTabularModel):
         """
         self._model = self._build_model()
 
-        import torch
-        if not self._cuda or not torch.cuda.is_available():
-            device = 'cpu'
-        elif isinstance(self._cuda, str):
-            device = self._cuda
-        else:
-            device = 'cuda'
+        categoricals = []
+        fields_before_transform = self._metadata.get_fields()
+        for field in table_data.columns:
+            if field in fields_before_transform:
+                meta = fields_before_transform[field]
+                if meta['type'] == 'categorical':
+                    categoricals.append(field)
 
-        self._model.device = torch.device(device)
+            else:
+                field_data = table_data[field].dropna()
+                if set(field_data.unique()) == {0.0, 1.0}:
+                    # booleans encoded as float values must be modeled as bool
+                    field_data = field_data.astype(bool)
 
-        categoricals = [
-            field
-            for field, meta in self._metadata.get_fields().items()
-            if meta['type'] == 'categorical'
-        ]
+                dtype = field_data.infer_objects().dtype
+                try:
+                    kind = np.dtype(dtype).kind
+                except TypeError:
+                    # probably category
+                    kind = 'O'
+                if kind in ['O', 'b']:
+                    categoricals.append(field)
 
         self._model.fit(
             table_data,
             discrete_columns=categoricals
         )
 
-    def _sample(self, num_rows):
+    def _sample(self, num_rows, conditions=None):
         """Sample the indicated number of rows from the model.
 
         Args:
             num_rows (int):
                 Amount of rows to sample.
+            conditions (dict):
+                If specified, this dictionary maps column names to the column
+                value. Then, this method generates `num_rows` samples, all of
+                which are conditioned on the given variables.
 
         Returns:
             pandas.DataFrame:
                 Sampled data.
         """
-        return self._model.sample(num_rows)
+        if conditions is None:
+            return self._model.sample(num_rows)
+
+        raise NotImplementedError(f"{self._MODEL_CLASS} doesn't support conditional sampling.")
 
 
 class CTGAN(CTGANModel):
@@ -136,6 +149,9 @@ class CTGAN(CTGANModel):
             Whether to have print statements for progress results. Defaults to ``False``.
         epochs (int):
             Number of training epochs. Defaults to 300.
+        pac (int):
+            Number of samples to group together when applying the discriminator.
+            Defaults to 10.
         cuda (bool or str):
             If ``True``, use CUDA. If a ``str``, use the indicated device.
             If ``False``, do not use cuda at all.
@@ -148,7 +164,7 @@ class CTGAN(CTGANModel):
                  embedding_dim=128, generator_dim=(256, 256), discriminator_dim=(256, 256),
                  generator_lr=2e-4, generator_decay=1e-6, discriminator_lr=2e-4,
                  discriminator_decay=0, batch_size=500, discriminator_steps=1,
-                 log_frequency=True, verbose=False, epochs=300, cuda=True):
+                 log_frequency=True, verbose=False, epochs=300, pac=10, cuda=True):
         super().__init__(
             field_names=field_names,
             primary_key=primary_key,
@@ -171,11 +187,11 @@ class CTGAN(CTGANModel):
             'discriminator_steps': discriminator_steps,
             'log_frequency': log_frequency,
             'verbose': verbose,
-            'epochs': epochs
+            'epochs': epochs,
+            'pac': pac,
+            'cuda': cuda
         }
 
-        self._cuda = cuda
-
 
 class TVAE(CTGANModel):
     """Model wrapping ``TVAESynthesizer`` model.
@@ -219,9 +235,9 @@ class TVAE(CTGANModel):
             arguments or learned from the data.
         embedding_dim (int):
             Size of the random sample passed to the Generator. Defaults to 128.
-        compress_dim (tuple or list of ints):
+        compress_dims (tuple or list of ints):
             Size of each hidden layer in the encoder. Defaults to (128, 128).
-        decompress_dim (tuple or list of ints):
+        decompress_dims (tuple or list of ints):
            Size of each hidden layer in the decoder. Defaults to (128, 128).
         l2scale (int):
             Regularization term. Defaults to 1e-5.
@@ -229,6 +245,8 @@ class TVAE(CTGANModel):
             Number of data samples to process in each step.
         epochs (int):
             Number of training epochs. Defaults to 300.
+        loss_factor (int):
+            Multiplier for the reconstruction error. Defaults to 2.
         cuda (bool or str):
             If ``True``, use CUDA. If a ``str``, use the indicated device.
             If ``False``, do not use cuda at all.
@@ -239,7 +257,7 @@ class TVAE(CTGANModel):
     def __init__(self, field_names=None, field_types=None, field_transformers=None,
                  anonymize_fields=None, primary_key=None, constraints=None, table_metadata=None,
                  embedding_dim=128, compress_dims=(128, 128), decompress_dims=(128, 128),
-                 l2scale=1e-5, batch_size=500, epochs=300, cuda=True):
+                 l2scale=1e-5, batch_size=500, epochs=300, loss_factor=2, cuda=True):
         super().__init__(
             field_names=field_names,
             primary_key=primary_key,
@@ -256,7 +274,7 @@ class TVAE(CTGANModel):
             'decompress_dims': decompress_dims,
             'l2scale': l2scale,
             'batch_size': batch_size,
-            'epochs': epochs
+            'epochs': epochs,
+            'loss_factor': loss_factor,
+            'cuda': cuda
         }
-
-        self._cuda = cuda
