@@ -5,8 +5,7 @@ from unittest.mock import Mock, call, patch
 import pandas as pd
 import pytest
 
-from sdv.metadata.dataset import (
-    Metadata, MetadataError, _load_csv, _parse_dtypes, _read_csv_dtypes)
+from sdv.metadata.dataset import Metadata, _load_csv, _parse_dtypes, _read_csv_dtypes
 
 
 def test__read_csv_dtypes():
@@ -369,8 +368,10 @@ class TestMetadata(TestCase):
         metadata._DTYPES = Metadata._DTYPES
 
         # Run
-        with pytest.raises(MetadataError):
-            Metadata.get_dtypes(metadata, 'test')
+        errors = []
+        Metadata.get_dtypes(metadata, 'test', errors=errors)
+
+        assert len(errors) == 1
 
     def test_get_dtypes_error_subtype_numerical(self):
         """Test get data types with an invalid numerical subtype."""
@@ -385,8 +386,10 @@ class TestMetadata(TestCase):
         metadata._DTYPES = Metadata._DTYPES
 
         # Run
-        with pytest.raises(MetadataError):
-            Metadata.get_dtypes(metadata, 'test')
+        errors = []
+        Metadata.get_dtypes(metadata, 'test', errors=errors)
+
+        assert len(errors) == 1
 
     def test_get_dtypes_error_subtype_id(self):
         """Test get data types with an invalid id subtype."""
@@ -401,8 +404,10 @@ class TestMetadata(TestCase):
         metadata._DTYPES = Metadata._DTYPES
 
         # Run
-        with pytest.raises(MetadataError):
-            Metadata.get_dtypes(metadata, 'test', ids=True)
+        errors = []
+        Metadata.get_dtypes(metadata, 'test', ids=True, errors=errors)
+
+        assert len(errors) == 1
 
     def test__get_pii_fields(self):
         """Test get pii fields"""
@@ -590,27 +595,45 @@ class TestMetadata(TestCase):
         assert result == 'a_primary_key'
         metadata.get_table_meta.assert_called_once_with('test')
 
-    def test_get_foreign_key(self):
+    def test_get_foreign_keys(self):
         """Test get foreign key"""
         # Setup
-        fields = {
-            'a_field': {
-                'ref': {
-                    'table': 'parent',
-                    'field': 'a_primary_key'
+        metadata = Metadata({
+            'tables': {
+                'parent': {
+                    'fields': {
+                        'parent_id': {
+                            'type': 'id',
+                        }
+                    },
+                    'primary_key': 'parent_id'
                 },
-                'name': 'a_field'
+                'child': {
+                    'fields': {
+                        'parent_id': {
+                            'type': 'id',
+                            'ref': {
+                                'table': 'parent',
+                                'field': 'id'
+                            }
+                        },
+                        'parent_id_2': {
+                            'type': 'id',
+                            'ref': {
+                                'table': 'parent',
+                                'field': 'id'
+                            }
+                        },
+                    }
+                }
             }
-        }
-        metadata = Mock(spec_set=Metadata)
-        metadata.get_fields.return_value = fields
+        })
 
         # Run
-        result = Metadata.get_foreign_key(metadata, 'parent', 'child')
+        result = Metadata.get_foreign_keys(metadata, 'parent', 'child')
 
         # Asserts
-        assert result == 'a_field'
-        metadata.get_fields.assert_called_once_with('child')
+        assert result == ['parent_id', 'parent_id_2']
 
     def test_reverse_transform(self):
         """Test reverse transform"""
